// hds header_start
//
// renoir header_end
`resetall
`timescale 1ns/10ps

`define load_skew       1
`define firmware_id_reg 3   // read firmware ID reg read only = UserCode
`define lm82_cntl       5   // LM82 control register
`define trig_int_st		6
`define dataout      	7
`define addrout      	8
`define link_reg		9
`define vfe_type_reg	10
`define soft_rst        12
`define fifo_rst        13
`define lm82_status    	27   // LM82 I2C data in
`define fifo_data_count 28
`define datain          29
`define trig_data_reg	30

module fe_command_decoder( 
   // Internal Declarations
   clk, 
   rst, 
   command_in, 
   soft_rstH, 
   fifo_rstH, 
   command_out, 
   data_valid, 
   command_real, 
   firmware_id_reg,  // read user code
   lm82_cntl,        // LM82 control register
   lm82_status,      // read LM82 I2C data in
   lm82_i2c_en, 
   addrout, 
   addrout_en, 
   datain, 
   datain_rd, 
   fifo_data_count,
   dataout, 
   dataout_wr,
   link_reg,  // link array
   vfe_type_reg,  // vfe tpe
   trig_int_st,
   trig_data_reg,
   trig_reg_en

);

// synopsys template
parameter DESIGNATOR   = 5,
          LENGTH_FIELD = 16,
          RESET_LENGTH = 16;

// Internal Declarations

input         clk;
input         rst;
input         command_in;
output        soft_rstH;
output        fifo_rstH;
output        command_out;
output        data_valid;
output        command_real;
input  [31:0] firmware_id_reg;
input  [31:0] fifo_data_count;
output [16:0] lm82_cntl;
input  [15:0] lm82_status;
input  [31:0] link_reg;
input  [31:0]  vfe_type_reg;
output        lm82_i2c_en;
output [9:0]  addrout;
output        addrout_en;
input  [31:0] datain;
output        datain_rd;
output [31:0] dataout;
output        dataout_wr;
output   	  trig_int_st;
input [31:0]  trig_data_reg;
output 		  trig_reg_en;


reg [LENGTH_FIELD-1:0] length;
reg r_wB;
reg [DESIGNATOR-1:0] designator;
reg [1:0] state;
reg data_valid;

always @(posedge clk or posedge rst)
begin
	if (rst)
		begin
		state <= #1 0;
		r_wB <= #1 0;
		length <= #1 0;
		designator <= #1 0;
		data_valid <= #1 0;
		end
	else
	begin
		case (state)
		0:
		begin
		// Shift in the command stream until a Designator is recognised
		r_wB <= #1 designator[DESIGNATOR-1];						 // assign bit 4
		designator <= #1 {designator[DESIGNATOR-2:0], command_in};   // assign bits 3 : 0  ie the designator
		data_valid <= #1 0;
		if (r_wB)		    // upon detection of first '1' go into state 1
			begin
			state <= #1 1;
			end
		end
		1:
		begin
			length <= #1 {length[LENGTH_FIELD-2:0], command_in};	   // length indicates the no. of data bits to follow
			data_valid <= #1 0;
			if (length[LENGTH_FIELD-1])	   // find the first '1'
				begin
				state <= #1 2;
				end
		end
		2:								   // stay in state 2 for the length number of clock cycles 
		begin							   // which equals the number of data bits to follow
			length <= #1 length - 1;
			data_valid <= #1 0;
			if (length == 1)
			begin
				if (r_wB)				   // if read cycle then reset everything
					begin
					state <= #1 0;
					designator <= #1 0;
					r_wB <= #1 0;
					end
				else					   // if write cycle then assert data valid
					begin
					state <= #1 3;
					designator <= #1 0;
					r_wB <= #1 0;
					end
			end
		end
		3:
		begin
			data_valid <= #1 1;   // assign a 1 to data valid
			state <= #1 0;
		end
		endcase
	end
end
// end of process




// these are strobe signals for loading the registers
reg soft_rst_int;
reg fifo_rst_int;
reg load_firmware_id_reg1, load_firmware_id_reg2, load_firmware_id_reg3;
reg load_lm82_cntl_1, load_lm82_cntl_2, load_lm82_cntl_3;	  		// two copies of original 
reg load_lm82_status_reg1, load_lm82_status_reg2, load_lm82_status_reg3;
reg load_dataout_1, load_dataout_2, load_dataout_3;	  		// two copies of original 
reg load_datain_1, load_datain_2, load_datain_3;	  		// 3 copies of original 
reg load_addrout_1, load_addrout_2, load_addrout_3;	  		// two copies of original 
reg trig_int_st;
reg   load_vfe_type_reg1, load_vfe_type_reg2, load_vfe_type_reg3;
reg   load_fifo_cnt_reg1, load_fifo_cnt_reg2, load_fifo_cnt_reg3;
reg   load_link_reg1, load_link_reg2, load_link_reg3;
reg   load_fakev_reg_en;
reg   load_trig_data_reg1, load_trig_data_reg2, load_trig_data_reg3;

// this process generates the relevant load strobe
always @(posedge clk or posedge rst)
begin
	if (rst)
		begin

		trig_int_st <= #1 0;
		soft_rst_int <= #1 0;
		fifo_rst_int <= #1 0;
		load_firmware_id_reg1 <= #1 0;
		load_firmware_id_reg2 <= #1 0;
		load_firmware_id_reg3 <= #1 0;
		load_lm82_cntl_1 <= #1 0;
		load_lm82_cntl_2 <= #1 0;
		load_lm82_cntl_3 <= #1 0;
		load_lm82_status_reg1 <= #1 0;
		load_lm82_status_reg2 <= #1 0;
		load_lm82_status_reg3 <= #1 0;
		load_datain_1 <= #1 0;
		load_datain_2 <= #1 0;
		load_datain_3 <= #1 0;
		load_dataout_1 <= #1 0;
		load_dataout_2 <= #1 0;
		load_dataout_3 <= #1 0;
		load_addrout_1 <= #1 0;
		load_addrout_2 <= #1 0;
		load_addrout_3 <= #1 0;

   		load_vfe_type_reg1 <= #1 0;
   		load_vfe_type_reg2 <= #1 0;
   		load_vfe_type_reg3 <= #1 0;

   		load_fifo_cnt_reg1 <= #1 0;
   		load_fifo_cnt_reg2 <= #1 0;
   		load_fifo_cnt_reg3 <= #1 0;

   		load_link_reg1 <= #1 0;
   		load_link_reg2 <= #1 0;
   		load_link_reg3 <= #1 0;

   		load_fakev_reg_en <= #1 0;

		load_trig_data_reg1 <=#1 0;
		load_trig_data_reg2 <=#1 0;
		load_trig_data_reg3 <=#1 0;

		end
	else
		begin
		soft_rst_int <= #1 (state == 2) & (designator == `soft_rst);
		fifo_rst_int <= #1 (state == 2) & (designator == `fifo_rst);
		load_firmware_id_reg1 <= #1 (state == 2) & (designator == `firmware_id_reg);
		load_firmware_id_reg2 <= #1 load_firmware_id_reg1;
		load_firmware_id_reg3 <= #1 load_firmware_id_reg2;
		load_vfe_type_reg1 <= #1 (state == 2) & (designator == `vfe_type_reg);
		load_vfe_type_reg2 <= #1 load_vfe_type_reg1;
		load_vfe_type_reg3 <= #1 load_vfe_type_reg2;
		load_fifo_cnt_reg1 <= #1 (state == 2) & (designator == `fifo_data_count);
		load_fifo_cnt_reg2 <= #1 load_fifo_cnt_reg1;
		load_fifo_cnt_reg3 <= #1 load_fifo_cnt_reg2;
		load_link_reg1 <= #1 (state == 2) & (designator == `link_reg);
		load_link_reg2 <= #1 load_link_reg1;
		load_link_reg3 <= #1 load_link_reg2;
		load_trig_data_reg1 <= #1 (state == 2) & (designator == `trig_data_reg);
		load_trig_data_reg2 <= #1 load_trig_data_reg1;
		load_trig_data_reg3 <= #1 load_trig_data_reg2;
		load_lm82_cntl_1 <= #1 (state == 2) & (designator == `lm82_cntl);
		load_lm82_cntl_2 <= #1 load_lm82_cntl_1;
		load_lm82_cntl_3 <= #1 load_lm82_cntl_2;
		load_lm82_status_reg1 <= #1 (state == 2) & (designator == `lm82_status);
		load_lm82_status_reg2 <= #1 load_lm82_status_reg1;
		load_dataout_1 <= #1 (state == 2) & (designator == `dataout);
		load_dataout_2 <= #1 load_dataout_1;
		load_dataout_3 <= #1 load_dataout_2;
		load_datain_1 <= #1 (state == 2) & (designator == `datain);
		load_datain_2 <= #1 load_datain_1;
		load_datain_3 <= #1 load_datain_2;
		load_addrout_1 <= #1 (state == 2) & (designator == `addrout);
		load_addrout_2 <= #1 load_addrout_1;
		load_addrout_3 <= #1 load_addrout_2;
		trig_int_st <= #1 (state == 2) & (designator == `trig_int_st);
		end
end
assign #1 fifo_rstH = fifo_rst_int ;
// generate RESET pulse
reg soft_rst_int2;
always @(negedge clk or posedge rst)
begin
	if (rst)
	begin
		soft_rst_int2 <= #1 0;
	end
	else
	begin
		soft_rst_int2 <= #1 soft_rst_int;
	end
end
assign #1 soft_rstH = (soft_rst_int2 | rst);

// new process which clocks data in and out of the registers
// define output registers here
reg [31:0] firmware_id_shift_reg;
reg [31:0]  vfe_type_shift_reg;
reg [31:0]  fifocnt_shift_reg;
reg [31:0] link_shift_reg;
reg [31:0] trig_data_shift_reg;
reg [15:0] lm82_status_shift_reg;
reg [16:0] lm82_cntl;			// LM82 I2C interface
reg lm82_i2c_en;
reg [31:0] datain_shift_reg;
reg [31:0] dataout;			// DATA OUT interface
reg [9:0] addrout;			// ADDRESS OUT interface
reg datain_rd;
reg dataout_wr;
reg addrout_en;
reg trig_reg_en;
always @(posedge clk or posedge rst)
begin
	if (rst)
	begin
		firmware_id_shift_reg <= #1 0;	  //user code eg: 0x12000211 
		lm82_cntl <= #1 0;
		lm82_i2c_en <= #1 0;
		trig_reg_en <= #1 0;
		lm82_status_shift_reg <= #1 0;
		datain_shift_reg <= #1 0;
		vfe_type_shift_reg <= #1 0;
		fifocnt_shift_reg <= #1 0;
		link_shift_reg <= #1 0;
		trig_data_shift_reg <= #1 0;
		datain_rd <= #1 0;
		dataout_wr <= #1 0;
		addrout_en <=#1 0;
		addrout <=#1 0;
		dataout <=#1 0;
	end
	else
	begin

		if (load_firmware_id_reg1 & !load_firmware_id_reg2) 
			begin
			firmware_id_shift_reg <= #1 firmware_id_reg;
			end
		if (load_firmware_id_reg2)
			begin
			firmware_id_shift_reg <= #1 {firmware_id_shift_reg[30:0], command_real};
			end

		if (load_vfe_type_reg1 & !load_vfe_type_reg2) 
			begin
			vfe_type_shift_reg <= #1 vfe_type_reg;
			end
		if (load_vfe_type_reg2)
			begin
			vfe_type_shift_reg <= #1 {vfe_type_shift_reg[30:0], command_real};
			end
//
		if (load_fifo_cnt_reg1 & !load_fifo_cnt_reg2) 
			begin
			fifocnt_shift_reg <= #1 fifo_data_count;
			end
		if (load_fifo_cnt_reg2)
			begin
			fifocnt_shift_reg <= #1 {fifocnt_shift_reg[30:0], command_real};
			end
//
		if (load_link_reg1 & !load_link_reg2) 
			begin
			link_shift_reg <= #1 link_reg;
			end
		if (load_link_reg2)
			begin
			link_shift_reg <= #1 {link_shift_reg[30:0], command_real};
			end
//
		if (load_trig_data_reg2 & !load_trig_data_reg3) 
			begin
			trig_data_shift_reg <= #1 trig_data_reg;
			end
		if (load_trig_data_reg3)
			begin
			trig_data_shift_reg <= #1 {trig_data_shift_reg[30:0], command_real};
			end
			trig_reg_en <=#1 (!load_trig_data_reg1 & load_trig_data_reg2);
//
		if (load_lm82_cntl_1)
			begin
			lm82_cntl <= #1 {lm82_cntl[15:0], command_real};
			end

			lm82_i2c_en <= #1 (!load_lm82_cntl_2 & load_lm82_cntl_3);	   // enable LM82 I2C interface 
//
		if (load_dataout_1)
			begin
			dataout <= #1 {dataout[30:0], command_real};
			end

			dataout_wr <= #1 (!load_dataout_1 & load_dataout_2);	   // enable DATA OUT interface
//
		if (load_datain_2 & !load_datain_3) 
			begin
			datain_shift_reg <= #1 datain;
			end
		if (load_datain_3)
			begin
			datain_shift_reg <= #1 {datain_shift_reg[30:0], command_real};
			end
		
		datain_rd <= #1 (load_datain_1 & !load_datain_2);	   // enable DATA IN interface
//
		if (load_addrout_1)
			begin
			addrout <= #1 {addrout[8:0], command_real};
			end

		addrout_en <= #1 (!load_addrout_1 & load_addrout_2);	   // enable ADDRESS OUT interface
//
		if (load_lm82_status_reg1 & !load_lm82_status_reg2) 
			begin
			lm82_status_shift_reg <= #1 lm82_status;
			end
		if (load_lm82_status_reg2)
			begin
			lm82_status_shift_reg <= #1 {lm82_status_shift_reg[14:0], command_real};
			end


		end
end


wire [31:0] command_word =
	{
	1'b0,			    // 31
	trig_data_shift_reg[31],	   		// 30
	datain_shift_reg[31],			    // 29
	fifocnt_shift_reg[31],			    // 28
	lm82_status_shift_reg[15],	// 27
	1'b0,				// 26
	1'b0,				// 25
	1'b0,				// 24
	1'b0,				// 23
	1'b0,				// 22
	1'b0,				// 21
	1'b0,			    // 20
	1'b0,				// 19
	1'b0,				// 18
	1'b0,				// 17
	1'b0,				// 16
	1'b0,				// 15
	1'b0,				// 14
	1'b0,				// 13
	1'b0,				// 12
	1'b0,				// 11
	vfe_type_shift_reg[31],		// 10
	link_shift_reg[31],			// 9
	addrout[9],	   			// 8
	dataout[31],			    // 7
	1'b0,				// 6
	lm82_cntl[16],			    // 5
	1'b0,				// 4
	firmware_id_shift_reg[31],	// 3
	1'b0,				// 2
	1'b0,						// 1
	1'b0};						// 0 unused

reg command_out, command_real ; 
always @(posedge clk or posedge rst)
begin
	if (rst)
	begin
		command_out <= #1 0;
		command_real <= #1 0;
	end
	else
	begin
		command_out <= #1 (state == 2) & command_word[designator];
		command_real <= #1 (r_wB ? command_word[designator] : command_in);
	end
end




// hds interface_end
endmodule